'
'   /\\\\\\\\\
'  /\\\///////\\\
'  \/\\\     \/\\\                                                 /\\\          /\\\
'   \/\\\\\\\\\\\/        /\\\\\     /\\\\\\\\\\     /\\\\\\\\   /\\\\\\\\\\\  /\\\\\\\\\\\  /\\\\\\\\\
'    \/\\\//////\\\      /\\\///\\\  \/\\\//////    /\\\/////\\\ \////\\\////  \////\\\////  \////////\\\
'     \/\\\    \//\\\    /\\\  \//\\\ \/\\\\\\\\\\  /\\\\\\\\\\\     \/\\\         \/\\\        /\\\\\\\\\\
'      \/\\\     \//\\\  \//\\\  /\\\  \////////\\\ \//\\///////      \/\\\ /\\     \/\\\ /\\   /\\\/////\\\
'       \/\\\      \//\\\  \///\\\\\/    /\\\\\\\\\\  \//\\\\\\\\\\    \//\\\\\      \//\\\\\   \//\\\\\\\\/\\
'        \///        \///     \/////     \//////////    \//////////      \/////        \/////     \////////\//
'                                  Let's find out together what makes a PIC Tick!
'
' Demonstrate Hardware PWM routines that make use of the extra CCP peripherals contained on some devices.
' It also allows frequencies upto 65535Hz
' Low end frequencies and high end frequencies depend on the oscillator value used.
' Frequencies may be approximate depending on the oscillator value used and the frequency required.
' This is not due to a limitation of the compiler, but of the peripheral itself.
'
' Note. All PWM peripherals will share a common frequency because they are all set to use Timer2.
'
' Written by Les Johnson for the Positron8 BASIC compiler.
' https://sites.google.com/view/rosetta-tech/home
'
    Device = 18F26K22                                   ' Tell the compiler what device to compile for
    Declare Xtal = 16                                   ' Tell the compiler what frequency the device will be operating at (in MHz)
'
' Setup which pins are used by the CCP peripherals
' Note that for some CCP port.pins, config fuse changes will be required
'
    Declare CCP1_Pin = PORTC.2
    Declare CCP2_Pin = PORTC.1
    Declare CCP3_Pin = PORTB.5
    Declare CCP4_Pin = PORTB.0
    Declare CCP5_Pin = PORTA.4

    Include "HPWM_18F.inc"                              ' Load the HPWM library into the program
'
' Create variables for the demo here
'
    Dim Duty      As Byte                               ' Holds the duty cycle value (0 to 255)
    Dim Frequency As Word                               ' Holds the frequency value

'---------------------------------------------------------------------------
Main:
'
' Adjust the frequency of all the CCP peripherals
'
(*
    For Frequency = 0 To 65535 Step 64                  ' Create a loop for frequency adjustment
        HPWM1_SetFreq(Frequency,127)                    ' Alter frequency with 50% duty cycle
        HPWM2_SetFreq(Frequency,127)                    ' Alter frequency with 50% duty cycle
        HPWM3_SetFreq(Frequency,127)                    ' Alter frequency with 50% duty cycle
        HPWM4_SetFreq(Frequency,127)                    ' Alter frequency with 50% duty cycle
        HPWM5_SetFreq(Frequency,127)                    ' Alter frequency with 50% duty cycle
        DelayMS 1
    Next
*)
'
' Adjust the duty cycles of all peripherals
'
    HPWM_SetFreq(2000,0)                                ' Set the frequency of all PWM peripherals

    HPWM1_Start()                                       ' \
    HPWM2_Start()                                       ' | Enable all peripherals
    HPWM3_Start()                                       ' |
    HPWM4_Start()                                       ' |
    HPWM5_Start()                                       ' /

    Do                                                  ' Create a loop
        For Duty = 0 To 255                             ' Create a duty cycle loop (0 to 255)
            HPWM1_Duty(Duty)                            ' \
            HPWM2_Duty(Duty)                            ' |
            HPWM3_Duty(Duty)                            ' | Adjust the duty cycle of the PWM
            HPWM4_Duty(Duty)                            ' |
            HPWM5_Duty(Duty)                            ' /
            DelayMS 10
        Next
    Loop                                                ' Do it forever


